# -*- coding: Latin-1 -*-

# Import arcpy module
import arcpy
import datetime
import math
import os
import sys
import subprocess

DATE_FORMAT = '%Y-%m-%d %H:%M:%S'
TIME_STAMP = datetime.datetime.now().strftime('%y%m%d_%H%M%S')
AI_COVERAGE_WS = "ARCINFO_WORKSPACE"
SHAPE_WS = "SHAPEFILE_WORKSPACE"
FILE_GDB_WS = "FILEGDB_WORKSPACE"

## Writes a message to the Python window aswell as the Tool execution dialog in ArcGIS
# @staticmethod
def write_message(s):
    arcpy.AddMessage(s)

def get_copy_name(name):
    test_name = '%s.mxd' % name
    i = 0
    
    while os.path.exists(test_name):
        i += 1
        test_name = '%s%d.mxd' % (name, i)

    return test_name

def RunToolLogic(mxd, newMxd, source_data_folder, import_2_filegdb, filegdb_name):

    file_gdb = import_2_filegdb.lower() == 'true'
    
    #####################################################################################################
    # Sektion dr mapp med nytt data skannas och lagras i ett dictionary objekt

    suffix = ''
    anno_dict = {}
    shape_dict = {}
    
    in_directory = os.walk(source_data_folder)
    first_directory = in_directory.next() # Hmtar den frsta mappen (huvudmappen)
    sub_folders = first_directory[1] # Listar alla undermappar. 

    
    # Hittar alla undermappar som har ett prefix p tv bokstver och ett '_' som tredje bokstav.
    # Dessa r coverage-filer
    coverages = [cov for cov in sub_folders if cov[:3].endswith('_')]

    # Loopar genom Coverage-filerna och stter annotation class in i ett dictionary och med
    # tvbokstavsprefixet som nyckel
    for cov in coverages:
        anno_dict[cov[:3]] = 'annotation.%s' % cov # Add to dictionary
   
    files = first_directory[2] # Hmtar alla filer i huvudmappen
    
    # Hmtar shape-filer som har ett tvbokstavsprefix och tredje bokstaven r '_' 
    shape_files = [f for f in files if f.endswith('.shp') and f[:3].endswith('_')]

    # Loopar igenom shape-filerna och lgger in dom i ett dictionary
    for f in shape_files:
        shape_dict[f[:3]] = f[:-4] # Add to dictionary
        if len(f) > 7: suffix = f[3:-4]

    #####################################################################################################
    # I den hr delen importeras shapefilerna in i en filgeodatabas
    if file_gdb:
        if not filegdb_name and suffix: # Om inte ngon filgeodatabas r angiven skapas en filgeodatabas i mappen fr klldata
            filegdb_name = os.path.join(source_data_folder, '%s.gdb' % suffix)
            write_message(u"Ingen filgeodatabas har angetts!")
        elif not filegdb_name:
            filegdb_name = os.path.join(source_data_folder, '%s.gdb' % os.path.basename(source_data_folder))
        if not filegdb_name.endswith('.gdb'):
            filegdb_name = '%s.gdb' % filegdb_name
            
        if not os.path.exists(filegdb_name): # Om inte filgeodatabasen finns skapas en ny
            write_message('Skapar filgeodatabas %s' % filegdb_name)
            arcpy.CreateFileGDB_management(os.path.dirname(filegdb_name), os.path.basename(filegdb_name))
                                           
        for key, item in shape_dict.items():
            if arcpy.Exists(os.path.join(filegdb_name, shape_dict[key])):
                write_message('Geoobjektsklassen %s finns redan i geodatabasen! Ingen import sker!' % shape_dict[key])
            else:
                write_message('Importerar %s.shp till geodatabas' % shape_dict[key])
                arcpy.FeatureClassToFeatureClass_conversion(os.path.join(source_data_folder, '%s.shp' % shape_dict[key]), filegdb_name, shape_dict[key])



    ####################################################################################
    # I den hr delen testas kartdokumentet om det finns ngot lager som behver byta dataklla
    layers_exists = False
    
    # Loopar alla lager i mxd-dokumentet
    for lyr in arcpy.mapping.ListLayers(mxd):

        # Kollar s att lagret stdjer metoden DATASOURCE                                                               
        if lyr.supports("DATASOURCE"):
            source_file = os.path.basename(lyr.dataSource) # Lagrar den gamla datakllans namn

            # Testar om datakllan r en coverage annotation
            # I s fall s r prefix bokstav 11, 12 och 13 i datakllans namn
            # Om inte s r prefix de tre frsta bokstverna i datakllans namn
            if source_file.startswith("annotation."):
                source_prefix = source_file[11:14] 
                new_fc = anno_dict.get(source_prefix, 'Err')
            else:
                source_prefix = source_file[:3]
                new_fc = shape_dict.get(source_prefix, 'Err')
            if new_fc != 'Err':
                layers_exists = True
                break # Om en dataklla finns att byta s avbryter vi testen hr
                
    ####################################################################################
    # I den hr delen byts alla datakllorna ut       

    if not layers_exists:
        arcpy.AddError('Inga lager i kartdockumentet och datakllor matchar varandra!')
        arcpy.AddError('Inga byten av datakllor har gjorts!')
        return
    
    #mxd_copy_name = get_copy_name(u'%s_lagade_skvgar' % mxd.filePath[:-4])
    
    write_message(u'Skapar en kopia av MXD-filen som ska rttas: %s' % newMxd)
    mxd.saveACopy(newMxd) # Sparar en kopia av kartdokumentet

    mxd2 = arcpy.mapping.MapDocument(newMxd) # Det r kopian som ska uppdateras med nya skvgar

    # Loopar alla lager i mxd-dokumentet
    for lyr in arcpy.mapping.ListLayers(mxd2):

        # Kollar s att lagret stdjer metoden DATASOURCE                                                               
        if lyr.supports("DATASOURCE"):
            source_file = os.path.basename(lyr.dataSource) # Lagrar den gamla datakllans namn

            # Testar om datakllan r en coverage annotation
            # I s fall s r prefix bokstav 11, 12 och 13 i datakllans namn
            # Om inte s r prefix de tre frsta bokstverna i datakllans namn
            if source_file.startswith("annotation."):
                source_prefix = source_file[11:14] 
                ws_type = AI_COVERAGE_WS
                new_fc = anno_dict.get(source_prefix, 'Err')
                workspace = source_data_folder
            elif file_gdb:
                source_prefix = source_file[:3]
                ws_type = FILE_GDB_WS
                new_fc = shape_dict.get(source_prefix, 'Err')
                workspace = filegdb_name
            else:
                source_prefix = source_file[:3]
                ws_type = SHAPE_WS
                new_fc = shape_dict.get(source_prefix, 'Err')
                workspace = source_data_folder

            if new_fc != 'Err': # Om ny dataklla finns s sker ett byte
                write_message('Byter datakallan for lager: %s' % source_prefix[:2])
                lyr.replaceDataSource (workspace, ws_type, new_fc)

    write_message(u'Sparar rttad mxd-fil!')
  
    mxd2.save()
   

class Toolbox(object):
    def __init__(self):
        # Define the toolbox (the name of the toolbox is the name of the .pyt file).
        
        self.label = "GSD MXD Administrator"
        self.alias = "GSD MXD Administrator"

        # List of tool classes associated with this toolbox
        self.tools = [GSDSourceAdministrator]


class GSDSourceAdministrator(object):
    def __init__(self):
        # Define the tool (tool name is the name of the class)
        self.label = "GSD Data Source Administrator"
        self.description = u"GSD Data Source Administrator uppdaterar datakllor fr lager i en GSD mxd-fil levererad av Lantmteriet." + \
                           u"De nya datakllorna utgrs av shape-filer och ArcINFO Coverage-filer" + \
                           u"som alla samlats i en och samma filmapp. " + \
                           u"Det finns en valfri mjlighet att i verktyget importera shape-filerna" + \
                           u"till en filgeodatabas och datakllan stts d till denna." + \
                           u"Karttexter i ArcINFO Coverage importeras inte till filgeodatabasen." 

                           
        self.canRunInBackground = False

    def getParameterInfo(self):
        #Definition av parametrar

        # Bool fr att anvnda det aktuella mxd-dokumentet
        use_current = arcpy.Parameter(
            displayName="Anvnd aktuellt MXD-dokument",
            name="use_current",
            datatype="GPBoolean",
            parameterType="Required",
            direction="Input")

        # Testar om verktyget exekveras av ArcMap eller ArcCatalog 
        if arcpy.sys.executable.lower().endswith('arcmap.exe'):
            use_current.value = True
            use_current.enabled = True
        else: # Om ArcCatalog exekverar kan inte aktuell mxd anvndas
            use_current.value = False
            use_current.enabled = False

        # Input ArcMap document
        mxd_doc = arcpy.Parameter(
            displayName="ArcMap MXD-fil",
            name="mxd_doc",
            datatype="DEMapDocument",
            parameterType="Required",
            direction="Input")
        if use_current.enabled:
            mxd = arcpy.mapping.MapDocument("CURRENT")
            mxd_doc.value = mxd.filePath
            mxd_doc.enabled = False
	           
        # Skvgen till nytt data
        source_dir = arcpy.Parameter(
            displayName="Filmapp med shape-filer",
            name="source_dir",
            datatype="DEFolder",
            parameterType="Required",
            direction="Input")
        
        # Bool fr att importera klldata till geodatabas
        fgdb_import = arcpy.Parameter(
            displayName="Importera till filgeodatabas",
            name="fgdb_import",
            datatype="GPBoolean",
            parameterType="Required",
            direction="Input")
        fgdb_import.value = False
        
        # Geodatabasen klldata ska importeras till 
        new_fgdb = arcpy.Parameter(
            displayName="Filgeodatabas",
            name="new_fgdb",
            datatype="DEWorkspace",
            parameterType="Optional",
            direction="Input")
        
	new_fgdb.filter.list = ["Local Database"]
	
	# Output ArcMap document
        newMxd_doc = arcpy.Parameter(
            displayName="Spara till ArcMap MXD-fil",
            name="newMxd_doc",
            datatype="DEMapDocument",
            parameterType="Required",
            direction="Output") 

        parameters = [use_current, mxd_doc, source_dir, newMxd_doc, fgdb_import, new_fgdb]
        
        return parameters


    def isLicensed(self):
        # Set whether tool is licensed to execute
        return True

    def updateParameters(self, parameters):
        # Frndrar parametervrden och tkomst till parametrar beroende p
        # anvndarens frndringar

        parameters[4].enabled = parameters[3].value

        parameters[1].enabled = not parameters[0].value
        if parameters[0].value:
            mxd = arcpy.mapping.MapDocument("CURRENT")
            parameters[1].value = mxd.filePath

        return

    def updateMessages(self, parameters):
        # Modify the messages created by internal validation for each tool
        # parameter.  This method is called after internal validation
        return

    def execute(self, parameters, messages):
        # Hmtar input variabler:
        mxd_doc = parameters[1].valueAsText
	source_data_folder = parameters[2].valueAsText
        newMxd_doc = parameters[3].valueAsText
	import_2_filegdb = parameters[4].valueAsText
        filegdb_name = parameters[5].valueAsText

        # Om aktuellt kartdokument ska anvndas s stts mxd_doc till nyckelordet CURRENT
        if parameters[0].value: mxd_doc = "CURRENT"

        mxd = arcpy.mapping.MapDocument(mxd_doc) # Hmtar MXD-dokumentet

        # Kr verktyget
        RunToolLogic(mxd, newMxd_doc, source_data_folder, import_2_filegdb, filegdb_name)

        arcpy.RefreshTOC()
        arcpy.RefreshActiveView()

